@@ -32,11 +32,6 @@ class PinnedFavs {
     reload() {
         const {state, signals, settings} = this.params;
         const appSystem = state.trigger('getAppSystem');
-        if (signals.isConnected('changed::pinned-apps', settings)) {
-            signals.disconnect('changed::pinned-apps', settings);
-        }
-        let cb = () => this.onFavoritesChange();
-        signals.connect(settings, 'changed::pinned-apps', cb);
         this._favorites = [];
         let ids = [];
         ids = settings.getValue('pinned-apps');
@@ -150,6 +145,7 @@ class PinnedFavs {
         }
 
         this.saveFavorites();
+        this.onFavoritesChange();
         return true;
     }
 
@@ -171,6 +167,7 @@ class PinnedFavs {
         this.triggerUpdate(appId, false);
         this._favorites.splice(refFav, 1);
         this.saveFavorites();
+        this.onFavoritesChange();
         return true;
     }
 }
@@ -298,7 +295,7 @@ class GroupedWindowListApplet extends Applet.Applet {
         this.signals.connect(global.window_manager, 'switch-workspace', (...args) => this.onSwitchWorkspace(...args));
         this.signals.connect(global.screen, 'workspace-removed', (...args) => this.onWorkspaceRemoved(...args));
         this.signals.connect(global.screen, 'window-monitor-changed', (...args) => this.onWindowMonitorChanged(...args));
-        this.signals.connect(Main.panelManager, 'monitors-changed', (...args) => this.on_applet_instances_changed(...args));
+        this.signals.connect(Main.panelManager, 'monitors-changed', (...args) => this._onMonitorsChanged(...args));
         this.signals.connect(global.screen, 'window-skip-taskbar-changed', (...args) => this.onWindowSkipTaskbarChanged(...args));
         this.signals.connect(global.display, 'window-marked-urgent', (...args) => this.updateAttentionState(...args));
         this.signals.connect(global.display, 'window-demands-attention', (...args) => this.updateAttentionState(...args));
@@ -372,7 +369,7 @@ class GroupedWindowListApplet extends Applet.Applet {
         this.state.set({appletReady: true});
     }
 
-    on_applet_instances_changed(instance) {
+    _updateState(initialUpdate) {
         if (!this.state.appletReady) {
             return;
         }
@@ -380,13 +377,21 @@ class GroupedWindowListApplet extends Applet.Applet {
         this.numberOfMonitors = null;
         this.updateMonitorWatchlist();
 
-        if (instance && instance.instance_id === this.instance_id) {
+        if (initialUpdate) {
             this.onSwitchWorkspace();
         } else {
             this.refreshCurrentAppList();
         }
     }
 
+    on_applet_instances_changed(instance) {
+        this._updateState(instance?.instance_id === this.instance_id);
+    }
+
+    _onMonitorsChanged(panelManager) {
+        this._updateState(false);
+    }
+
     on_panel_edit_mode_changed() {
         this.state.set({panelEditMode: !this.state.panelEditMode});
         each(this.appLists, (workspace) => {
@@ -509,7 +514,7 @@ class GroupedWindowListApplet extends Applet.Applet {
 
     updateMonitorWatchlist() {
         if (!this.numberOfMonitors) {
-            this.numberOfMonitors = Gdk.Screen.get_default().get_n_monitors();
+            this.numberOfMonitors = global.display.get_n_monitors();
         }
         let onPrimary = this.panel.monitorIndex === Main.layoutManager.primaryIndex;
         let instances = Main.AppletManager.getRunningInstancesForUuid(this.state.uuid);
@@ -544,7 +549,7 @@ class GroupedWindowListApplet extends Applet.Applet {
 
     refreshCurrentAppList() {
         let appList = this.appLists[this.state.currentWs];
-        if (appList) setTimeout(() => appList.refreshList(), 0);
+        if (appList) appList.refreshList();
     }
 
     refreshAllAppLists() {
@@ -790,11 +795,18 @@ class GroupedWindowListApplet extends Applet.Applet {
         let pos = 0;
         while(pos < this.state.dragging.posList.length && axis[0] > this.state.dragging.posList[pos])
             pos++;
+        
+        let favLength = 0;
+        each(appList.appList, (appGroup, i) => {
+            if(appGroup.groupState.isFavoriteApp)
+                favLength++;
+            else return false;
+        });
 
         // keep pinned and unpinned items separate
-        if((this.state.dragging.isForeign && pos > this.pinnedFavorites._favorites.length) ||
-            (!this.state.dragging.isForeign && source.groupState.isFavoriteApp && pos >= this.pinnedFavorites._favorites.length) ||
-            (!this.state.dragging.isForeign && !source.groupState.isFavoriteApp && pos < this.pinnedFavorites._favorites.length))
+        if((this.state.dragging.isForeign && pos > favLength) ||
+            (!this.state.dragging.isForeign && source.groupState.isFavoriteApp && pos >= favLength) ||
+            (!this.state.dragging.isForeign && !source.groupState.isFavoriteApp && pos < favLength))
             return DND.DragMotionResult.NO_DROP;
 
         // handle position change
@@ -880,11 +892,11 @@ class GroupedWindowListApplet extends Applet.Applet {
     }
 
     moveLauncher(source) {
-        let pos = this.state.dragging.pos;
+        let appList = this.getCurrentAppList();
         this.clearDragParameters();
 
         Meta.later_add(Meta.LaterType.BEFORE_REDRAW, () => {
-            this.getCurrentAppList().updateAppGroupIndexes();
+            appList.updateAppGroupIndexes();
             // Refresh the group's thumbnails so hoverMenu is aware of the position change
             // In the case of dragging a group that has a delay before Cinnamon can grab its
             // thumbnail texture, e.g., LibreOffice, defer the refresh.
@@ -894,14 +906,26 @@ class GroupedWindowListApplet extends Applet.Applet {
 
             // Handle favoriting if pin on drag is enabled
             if (!source.groupState.app.is_window_backed()) {
-                let opts = {
-                    appId: source.groupState.appId,
-                    app: source.groupState.app,
-                    pos
-                };
+
                 let refFav = findIndex(this.pinnedFavorites._favorites, (favorite) => favorite.id === source.groupState.appId);
                 if (refFav > -1) {
-                    this.pinnedFavorites.moveFavoriteToPos(opts);
+                    
+                    let pinned = []; //pinned apps found before source
+                    each(appList.appList, (appGroup, i) => {
+                        if(appGroup.groupState.appId == source.groupState.appId)//
+                            return false;
+                        else if(!pinned.includes(appGroup.groupState.appId))
+                                pinned.push(appGroup.groupState.appId);
+                    });
+   
+                    let opts = {
+                        appId: source.groupState.appId,
+                        app: source.groupState.app,
+                        pos : pinned.length
+                    };
+
+                    if(pinned.length != refFav)
+                        this.pinnedFavorites.moveFavoriteToPos(opts);
                 }
             }
 
