@@ -10,7 +10,7 @@ class Module:
 
     def __init__(self, content_box):
         keywords = _("display, screen, monitor, layout, resolution, dual, lcd")
-        self.sidePage = SidePage(_("Display"), "cs-display", keywords, content_box, 570, module=self)
+        self.sidePage = SidePage(_("Display"), "cs-display", keywords, content_box, 650, module=self)
         self.display_c_widget = None
 
     def on_module_selected(self):
@@ -36,6 +36,7 @@ class Module:
 
                 self.display_c_widget = content
                 settings.add_row(widget)
+                widget.get_parent().set_activatable(False)
 
             except Exception as detail:
                 print(detail)
@@ -48,9 +49,41 @@ class Module:
             switch.set_tooltip_text(_("Select this option to disable automatic screen rotation on hardware equipped with supported accelerometers."))
             settings.add_row(switch)
 
-            switch = GSettingsSwitch(_("Enable fractional scaling controls (experimental)"), "org.cinnamon.control-center.display", "show-fractional-scaling-controls")
+            switch = Switch(_("Enable fractional scaling controls (experimental)"))
             switch.set_tooltip_text(_("Select this option to display additional layout controls for per-monitor scaling."))
             settings.add_row(switch)
+            self.fractional_switch = switch.content_widget
+
+            self.muffin_settings = Gio.Settings(schema_id="org.cinnamon.muffin")
+            self.experimental_features_changed(self.muffin_settings, "x11-randr-fractional-scaling")
+            self.muffin_settings.connect("changed::experimental-features", self.experimental_features_changed)
+            self.fractional_switch.connect("notify::active", self.fractional_switch_toggled)
+
+    def experimental_features_changed(self, settings, key):
+        self.fractional_switch.freeze_notify()
+
+        features = self.muffin_settings.get_strv("experimental-features")
+        self.fractional_switch.set_active("x11-randr-fractional-scaling" in features)
+
+        self.fractional_switch.thaw_notify()
+
+    def fractional_switch_toggled(self, switch, pspec):
+        active = switch.get_active()
+
+        features = self.muffin_settings.get_strv("experimental-features")
+
+        if active:
+            if "x11-randr-fractional-scaling" in features:
+                return
+            else:
+                features.append("x11-randr-fractional-scaling")
+        else:
+            try:
+                features.remove("x11-randr-fractional-scaling")
+            except ValueError:
+                pass
+
+        self.muffin_settings.set_strv("experimental-features", features)
 
     def on_navigate_out_of_module(self):
         if self.display_c_widget:
