@@ -38,6 +38,7 @@ const DESKLET_SNAP_KEY = 'desklet-snap';
 const DESKLET_SNAP_INTERVAL_KEY = 'desklet-snap-interval';
 const KEYBINDING_SCHEMA = 'org.cinnamon.desktop.keybindings';
 const SHOW_DESKLETS_KEY = 'show-desklets';
+const LOCK_DESKLETS_KEY = "lock-desklets";
 
 function initEnabledDesklets() {
     for (let i = 0; i < definitions.length; i++) {
@@ -60,15 +61,15 @@ function unloadRemovedDesklets(removedDeskletUUIDs) {
  *
  * Initialize desklet manager
  */
-function init(){
-    let startTime = new Date().getTime();
+function init() {
+    const startTime = new Date().getTime();
     try {
         desklets = imports.desklets;
     } catch (e) {
         desklets = {};
     }
     deskletMeta = Extension.Type.DESKLET.legacyMeta;
-    deskletsLoaded = false
+    deskletsLoaded = false;
 
     definitions = getDefinitions();
 
@@ -79,7 +80,7 @@ function init(){
 
         deskletsLoaded = true;
         updateMouseTracking();
-        global.log("DeskletManager started in " + new Date().getTime() - startTime + "ms");
+        global.log(`DeskletManager started in ${new Date().getTime() - startTime} ms`);
     });
 }
 
@@ -275,17 +276,20 @@ function _unloadDesklet(deskletDefinition, deleteConfig) {
 }
 
 function _removeDeskletConfigFile(uuid, instanceId) {
-    let config_path = (GLib.get_home_dir() + "/" +
-                               ".cinnamon" + "/" +
-                                 "configs" + "/" +
-                                      uuid + "/" +
-                                instanceId + ".json");
-    let file = Gio.File.new_for_path(config_path);
-    if (file.query_exists(null)) {
-        try {
-            file.delete(null);
-        } catch (e) {
-            global.logError("Problem removing desklet config file during cleanup.  UUID is " + uuid + " and filename is " + config_path);
+    let config_paths = [
+        [GLib.get_home_dir(), ".cinnamon", "configs", uuid, instanceId + ".json"].join("/"),
+        [GLib.get_user_config_dir(), "cinnamon", "spices", uuid, instanceId + ".json"].join("/")
+    ];
+
+    for (let i = 0; i < config_paths.length; i++) {
+        const config_path = array[i];
+        let file = Gio.File.new_for_path(config_path);
+        if (file.query_exists(null)) {
+            try {
+                file.delete(null);
+            } catch (e) {
+                global.logError("Problem removing desklet config file during cleanup.  UUID is " + uuid + " and filename is " + config_path);
+            }
         }
     }
 }
@@ -425,7 +429,7 @@ function DeskletContainer(){
 
 DeskletContainer.prototype = {
     _init: function(){
-        this.actor = new Clutter.Group();
+        this.actor = global.desklet_container;
         this.actor._delegate = this;
 
         this.last_x = -1;
@@ -445,6 +449,8 @@ DeskletContainer.prototype = {
                 this.lower();
             }
         });
+
+        global.settings.connect('changed::' + LOCK_DESKLETS_KEY, () => this.onDeskletsLockedChanged());
     },
 
     applyKeyBindings: function() {
@@ -463,6 +469,7 @@ DeskletContainer.prototype = {
      */
     addDesklet: function(actor){
         this.actor.add_actor(actor);
+        actor._delegate._draggable.inhibit = global.settings.get_boolean(LOCK_DESKLETS_KEY);
     },
 
     /**
@@ -477,6 +484,12 @@ DeskletContainer.prototype = {
         return this.actor.contains(actor);
     },
 
+    onDeskletsLockedChanged: function(settings, key) {
+        this.actor.get_children().forEach((deskletActor) => {
+            deskletActor._delegate._draggable.inhibit = global.settings.get_boolean(LOCK_DESKLETS_KEY);
+        });
+    },
+
     handleDragOver: function(source, actor, x, y, time) {
         deskletsDragging = true;
 
@@ -618,9 +631,8 @@ DeskletContainer.prototype = {
     handleStageEvent: function(actor, event) {
         let target = event.get_source();
         let type = event.type();
-
         if ((type === Clutter.EventType.BUTTON_PRESS || type === Clutter.EventType.BUTTON_RELEASE)
-            && target.toString().indexOf('ClutterStage') > -1) {
+            && target.get_parent() instanceof Meta.WindowActor) {
             this.lower();
         }
 
@@ -631,12 +643,12 @@ DeskletContainer.prototype = {
         if (this.actor.get_children().length === 0) {
             return;
         }
-        this.actor.get_parent().set_child_above_sibling(this.actor, null);
+        global.display.set_desklets_above(true);
         this.setModal();
     },
 
     lower: function() {
-        this.actor.get_parent().set_child_below_sibling(this.actor, global.window_group);
+        global.display.set_desklets_above(false);
         this.unsetModal();
     },
 
