@@ -59,7 +59,7 @@
 
 #include "st-entry.h"
 
-#include "st-im-text.h"
+// #include "st-im-text.h"
 #include "st-icon.h"
 #include "st-widget.h"
 #include "st-texture-cache.h"
@@ -225,12 +225,6 @@ st_entry_dispose (GObject *object)
       priv->blink_timeout = 0;
     }
 
-  if (priv->entry)
-    {
-      clutter_actor_destroy (priv->entry);
-      priv->entry = NULL;
-    }
-
   keymap = gdk_keymap_get_for_display (gdk_display_get_default ());
   g_signal_handlers_disconnect_by_func (keymap, keymap_state_changed, entry);
 
@@ -718,12 +712,14 @@ st_entry_clipboard_callback (StClipboard *clipboard,
                              const gchar *text,
                              gpointer     data)
 {
-  ClutterText *ctext = (ClutterText*)((StEntry *) data)->priv->entry;
+  ClutterText *ctext;
   gint cursor_pos;
 
   if (!text)
     return;
 
+  ctext = (ClutterText*)((StEntry *) data)->priv->entry;
+
   /* delete the current selection before pasting */
   clutter_text_delete_selection (ctext);
 
@@ -929,7 +925,7 @@ st_entry_init (StEntry *entry)
 
   priv = entry->priv = st_entry_get_instance_private (entry);
 
-  priv->entry = g_object_new (ST_TYPE_IM_TEXT,
+  priv->entry = g_object_new (CLUTTER_TYPE_TEXT,
                               "line-alignment", PANGO_ALIGN_LEFT,
                               "editable", TRUE,
                               "reactive", TRUE,
